//////////////////////////////////////////////////////////////////////
//
// Module    : AddReferenceDialog.cpp
// Author    : Krasimir Angelov <kr.angelov@gmail.com>
// Copyright : (c) Krasimir Angelov, All Rights Reserved
//
// Implementation of CAddReferenceDialog
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AddReferenceDialog.h"


// CAddReferenceDialog

CAddReferenceDialog::CAddReferenceDialog()
{
	m_pUIShell = NULL;
	m_pPackages = NULL;
}

CAddReferenceDialog::~CAddReferenceDialog()
{
	FreePackages();
}

void CAddReferenceDialog::FreePackages()
{
	while (m_pPackages)
	{
		PackageInfo *p = m_pPackages;
		m_pPackages = m_pPackages->pNext;
		free(p);
	}
}

LRESULT CAddReferenceDialog::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	CAxDialogImpl<CAddReferenceDialog>::OnInitDialog(uMsg, wParam, lParam, bHandled);

	if (m_pUIShell)
		m_pUIShell->CenterDialogOnWindow(m_hWnd, NULL);

	CWindow wndAvPackages = GetDlgItem(IDC_AVAILABLE_PACKAGES_LIST);

	PackageInfo *p = m_pPackages;
	while (p)
	{
		wndAvPackages.SendMessage(LB_ADDSTRING, 0, (LPARAM) p->szName);
		p = p->pNext;
	}

	return 1;  // Let the system set the focus
}

LRESULT CAddReferenceDialog::OnClickedOK(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	CWindow wndAvPackages = GetDlgItem(IDC_AVAILABLE_PACKAGES_LIST);

	int nCount = wndAvPackages.SendMessage(LB_GETSELCOUNT);
	int *buffer = (int *) malloc(sizeof(int)*nCount);
	nCount = wndAvPackages.SendMessage(LB_GETSELITEMS, nCount, (LPARAM) buffer);

	FreePackages();
	for (int i = 0; i < nCount; i++)
	{
		int nLen = wndAvPackages.SendMessage(LB_GETTEXTLEN, buffer[i]);

		PackageInfo *pPackage = (PackageInfo *) malloc(sizeof(PackageInfo)+nLen);
		wndAvPackages.SendMessage(LB_GETTEXT, buffer[i], (LPARAM) &pPackage->szName);
		pPackage->pNext = m_pPackages;
		m_pPackages = pPackage;
	}

	free(buffer);
	EndDialog(wID);
	return 0;
}

LRESULT CAddReferenceDialog::OnClickedCancel(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	FreePackages();
	EndDialog(wID);
	return 0;
}

STDMETHODIMP CAddReferenceDialog::OpenDialog(IServiceProvider *provider)
{
	HRESULT hr;

	hr = provider->QueryService(IID_IVsUIShell, IID_IVsUIShell, (void **) &m_pUIShell);
	if (FAILED(hr))
		return hr;

	HWND hOwnerWnd;
	hr = m_pUIShell->GetDialogOwnerHwnd(&hOwnerWnd);
	if (FAILED(hr))
		return hr;

	DoModal(hOwnerWnd);

	m_pUIShell->Release();
	m_pUIShell = NULL;

	return S_OK;
}

STDMETHODIMP CAddReferenceDialog::AddPackage(CHAR* name)
{
	PackageInfo *pPackage = (PackageInfo *) malloc(sizeof(PackageInfo)+strlen(name));
	pPackage->pNext = m_pPackages;
	strcpy(pPackage->szName, name);
	m_pPackages = pPackage;
	return S_OK;
}

STDMETHODIMP CAddReferenceDialog::GetPackage(PVOID * pos, CHAR** name)
{
	if (*pos)
		*pos  = ((PackageInfo*) (*pos))->pNext;
	else
		*pos = m_pPackages;

	if (*pos)
		*name = ((PackageInfo*) (*pos))->szName;
	else
		*name = "";

	return S_OK;
}
